/*
 \file app_common.h
 \brief Contains macros and inline functions common to all applications
 */
/*
 * Copyright (C) 2010,2011 Freescale Semiconductor, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _APP_COMMON_H
#define _APP_COMMON_H

#include <stdbool.h>

#define MAGIC(n) do {						\
	__asm__ __volatile__ ("rlwimi %0,%0,0,%1,%2"		 \
				:: "i" (((n) >> 10) & 0x1f),	   \
				"i" (((n) >>  5) & 0x1f),	\
				"i" (((n) >>  0) & 0x1f));	\
} while (0)

#define MAGIC_BREAKPOINT MAGIC(0)

#define JIFFY_PER_SEC		(100)	/**< Number of Jiffies per second */

#define CPU_JIFFY_CYCLES	(CPU_HZ / JIFFY_PER_SEC)
/**< Calculates Cycles per jiffy */

#define CPU_JIFFY_MASK		(32 - (__builtin_clz(CPU_JIFFY_CYCLES) - 1))
/**< Mask for reading only the bits that would change when a jiffy passes */

#define MAX_NUM_BMAN_POOLS 64
#undef ONE_MILLION_ROUTE_SUPPORT
#endif	/* APP_COMMON__H */
